'use strict';

import chai from 'chai';
import {Injectable} from '../../lib.compiled/Microservice/Injectable';
import {Instance} from '../../lib.compiled/Microservice/Instance';
import {MissingWorkingMicroserviceException} from '../../lib.compiled/Microservice/Exception/MissingWorkingMicroserviceException';
import Core from 'deep-core';

suite('Microservice/Injectable', function() {
  let injectable = new Injectable();

  test('Class Injectable exists in Microservice/Injectable', function() {
    chai.expect(typeof Injectable).to.equal('function');
  });

  test('Check constructor sets _config', function() {
    chai.expect(injectable._microservice).to.be.equal(null);
  });

  test('Check microservice getter throws \'MissingWorkingMicroserviceException\' ' +
    'exception for _microservice === null', function() {
    let error = null;
    let actualResult = null;
    try {
      actualResult = injectable.microservice;
    } catch (e) {
      error = e;
    }

    chai.expect(error).to.be.not.equal(null);
    chai.expect(error).to.be.an.instanceof(MissingWorkingMicroserviceException);
    chai.expect(error.message).to.be.equal('Missing working microservice from Kernel.MicroserviceInjectable');
  });

  test('Check microservice setter throws \'Core.Exception.InvalidArgumentException\' exception', function() {
    let error = null;
    let instance = 'invalidInstance';
    try {
      injectable.microservice = instance;
    } catch (e) {
      error = e;
    }

    chai.expect(error).to.be.not.equal(null);
  });

  test('Check microservice setter sets _microservice', function() {
    let error = null;
    let instance = null;
    try {
      instance = new Instance();
      injectable.microservice = instance;
    } catch (e) {
      error = e;
    }

    chai.expect(error).to.be.equal(null);
    chai.expect(injectable.microservice).to.be.equal(instance);
  });

  test('Check bind() method sets _microservice', function() {
    let error = null;
    let instance = null;
    try {
      instance = new Instance('identifier', 'rawResources');
      injectable.bind(instance);
    } catch (e) {
      error = e;
    }

    chai.expect(error).to.be.equal(null);
    chai.expect(injectable.microservice).to.be.equal(instance);
  });
});
